from geoformat.conversion.segment_conversion import segment_list_to_linestring

from tests.data.segments import (
    segment_a,
    segment_b,
    segment_c,
    segment_e,
    segment_h)

from test_all import test_function


segment_list_to_linestring_parameters = {
    0: {
        "segment_list": [segment_a, segment_b],
        "bbox": False,
        "segment_as_part": False,
        "return_value": {"type": "LineString", "coordinates": [[0, 0], [0, 1], [0, 0]]}
    },
    1: {
        "segment_list": [segment_a, segment_b, segment_c],
        "bbox": False,
        "segment_as_part": False,
        "return_value":  {"type": "LineString", "coordinates": [[0, 0], [0, 1], [0, 0], [0, -1]]}
    },
    2: {
        "segment_list": [segment_a, segment_c, segment_e, segment_h],
        "bbox": False,
        "segment_as_part": False,
        "return_value": {"type": "LineString",
                         "coordinates": [[0, 0], [0, 1], [0, 0], [0, -1], [-1, -1], [1, 1], [-1, -1], [-1, 1]]}
    },
    3: {
        "segment_list": [segment_a, segment_b],
        "bbox": False,
        "segment_as_part": True,
        "return_value": {"type": "MultiLineString", "coordinates": [[[0, 0], [0, 1]], [[0, 1], [0, 0]]]}
    },
    4: {
        "segment_list": [segment_a, segment_b, segment_c],
        "bbox": False,
        "segment_as_part": True,
        "return_value": {"type": "MultiLineString",
                         "coordinates": [[[0, 0], [0, 1]], [[0, 1], [0, 0]], [[0, 0], [0, -1]]]}
    },
    5: {
        "segment_list": [segment_a, segment_c, segment_e, segment_h],
        "bbox": False,
        "segment_as_part": True,
        "return_value": {"type": "MultiLineString",
                         "coordinates": [[[0, 0], [0, 1]], [[0, 0], [0, -1]], [[-1, -1], [1, 1]], [[-1, -1], [-1, 1]]]}
    },
    6: {
        "segment_list": [segment_a, segment_c, segment_e, segment_h],
        "bbox": True,
        "segment_as_part": False,
        "return_value": {"type": "LineString",
                         "coordinates": [[0, 0], [0, 1], [0, 0], [0, -1], [-1, -1], [1, 1], [-1, -1], [-1, 1]],
                         "bbox": (-1, -1, 1, 1)}
    },
    7: {
        "segment_list": [segment_a, segment_c, segment_e, segment_h],
        "bbox": True,
        "segment_as_part": True,
        "return_value": {"type": "MultiLineString",
                         "coordinates": [[[0, 0], [0, 1]], [[0, 0], [0, -1]], [[-1, -1], [1, 1]], [[-1, -1], [-1, 1]]],
                         "bbox": (-1, -1, 1, 1)}
    },
    8: {
        "segment_list": [segment_a],
        "bbox": False,
        "segment_as_part": True,
        "return_value": {"type": "LineString",
                         "coordinates": [[0, 0], [0, 1]],
                         }
    },
    9: {
        "segment_list": [segment_a],
        "bbox": True,
        "segment_as_part": True,
        "return_value": {"type": "LineString",
                         "coordinates": [[0, 0], [0, 1]],
                         "bbox": (0, 0, 0, 1)}
    },
}


def test_all():
    # segment_list_to_linestring
    print(test_function(segment_list_to_linestring, segment_list_to_linestring_parameters))


if __name__ == '__main__':
    test_all()



from geoformat.geoprocessing.geoparameters.boundaries import ccw_or_cw_boundary

from test_all import test_function

ccw_or_cw_boundary_parameters = {
    0: {
        "boundary_coordinates": [[-1, 1], [1, 1], [1, -1], [-1, -1]],  # convex boundary
        "return_value": 'CW'
    },
    1: {
        "boundary_coordinates": [[-1, -1], [1, -1], [1, 1], [-1, 1]],  # reverse convex boundary
        "return_value": 'CCW'
    },
    2: {
        "boundary_coordinates": [[-1, -1], [0, 0], [1, -1], [1, 1], [-1, 1]],  # concave boundary
        "return_value": 'CCW'
    },
    3: {
        "boundary_coordinates": [[-1, 1], [1, 1], [1, -1], [0, 0], [-1, -1]],  # reverse concave boundary
        "return_value": 'CW'
    }
}


def test_all():
    # ccw_or_cw_boundary
    print(test_function(ccw_or_cw_boundary, ccw_or_cw_boundary_parameters))


if __name__ == '__main__':
    test_all()


import csv

import geoformat
from geoformat.conf.path import verify_input_path_is_file
from geoformat.conversion.feature_conversion import feature_list_to_geolayer


def _from_csv_get_features_list(csv_reader, header):
    """
    Take csv reader object and yield csv row into geoformat feature

    :param csv_reader: csv reader object.
    :param header: Specifies that the file contains a header line with the names of each column in the file. (bool)

    :return: geoformat Feature
    """
    for i_row, row in enumerate(csv_reader):
        if i_row == 0:
            if header is True:
                field_name_list = list(row)
                continue
            else:
                field_name_list = ['field_{}'.format(i_field) for i_field in range(len(row))]

        feature = {}
        feature['attributes'] = {field_name: row[i_field] for i_field, field_name in enumerate(field_name_list)}
        yield feature


def csv_to_geolayer(
        path,
        delimiter=',',
        header=True,
        field_name_filter=None,
        force_field_conversion=False,
        serialize=False,
):
    """
    From csv file get a geolayer.

    :param path: path to csv file
    :param delimiter: Specifies the character that separates columns within each row (line) of the file.
     The default is comma character.
    :param header: Specifies that the file contains a header line with the names of each column in the file. (bool)
    :param field_name_filter: filter only on specified field_name (can be a list)
    :param force_field_conversion: True if you want to force value in field (can change field type) / False if you want
           to deduce field type without forcing field type.
    :param serialize: True if features in geolayer are serialized (can reduce performance) / False if not.

    :return: geolayer
    """
    # verify that path exist
    p = verify_input_path_is_file(path)

    with open(p, 'r') as csv_file:
        raw_feature_list = csv.reader(csv_file, delimiter=delimiter)

        raw_feature_list = list(_from_csv_get_features_list(csv_reader=raw_feature_list, header=header))

        geolayer = feature_list_to_geolayer(
            feature_list=raw_feature_list,
            geolayer_name=p.stem,
            field_name_filter=field_name_filter,
            force_field_conversion=force_field_conversion,
            crs=None,
            serialize=serialize
        )

        return geolayer



import types
from pathlib import Path

module_without_unit_test = {
    "geoformat/conf/decorator.py",
    "geoformat/conf/driver_variable.py",
    "geoformat/conf/error_messages.py",
    "geoformat/conf/fields_variable.py",
    "geoformat/conf/geometry_variable.py",
    "geoformat/conf/timer.py",
    "geoformat/db/db_request.py",
    "geoformat/driver/ogr/ogr_driver.py",
    "geoformat/explore_data/random_geometry.py"
}

deprecated_function = {'clause_where_combination'}


def iterate_over_directory_in_geoformat_module(p):
    # get dir
    dir_list = [x for x in p.iterdir() if x.is_dir() and x.name != '__pycache__']

    if dir_list:
        for dir in dir_list:
            dir_list += iterate_over_directory_in_geoformat_module(p=dir)

    return dir_list


def get_relative_path(path, origin_path):
    return '/'.join(path.parts[len(origin_path.parts):])


class GeoformatInspector:

    def __init__(self, path='None'):
        self.geoformat_root_path = Path(__file__).parent.parent.parent.joinpath()
        self.geoformat_lib_dir_path = self.geoformat_root_path.joinpath('geoformat')
        self.tests_dir_path = self.geoformat_root_path.joinpath('tests')
        self.tests_geoformat_lib_dir = self.geoformat_root_path.joinpath('tests/geoformat')

    def check_tests_structure(self):
        dir_list = iterate_over_directory_in_geoformat_module(p=self.geoformat_lib_dir_path)

        for dir_path in dir_list:
            relative_dir_part = get_relative_path(path=dir_path, origin_path=self.geoformat_lib_dir_path)
            relativ_part_path = Path(relative_dir_part)
            test_geoformat_lib_dir_path = self.tests_geoformat_lib_dir.joinpath(relativ_part_path)
            if test_geoformat_lib_dir_path.is_dir() is False:
                print('tests dir {} missing !'.format(relativ_part_path))

    def get_objects_in_geoformat_module(self, geoformat_module_import_path, from_list=None, object_type=None):

        if from_list is None:
            from_list = [None]
        function_set = set()

        try:
            module = __import__(
                name=geoformat_module_import_path,
                locals=None,
                globals=None,
                fromlist=from_list,
                level=0
            )

            for str_obj in dir(module):
                if not str_obj.startswith('_'):  # exclude function that start to '_'
                    obj = getattr(module, str_obj)

                    write_str_obj = True
                    if object_type:
                        if not isinstance(obj, object_type):
                            write_str_obj = False
                    if write_str_obj is True:
                        function_set.update({str_obj})

        except ModuleNotFoundError:
            parts_path = geoformat_module_import_path.split('.')
            file_path = Path(*parts_path)
            print('tests files {} missing'.format(file_path))

        return function_set

    def check_tests_function(self):
        dir_list = iterate_over_directory_in_geoformat_module(p=self.geoformat_lib_dir_path)
        geoformat_function_to_test_set = set()
        geoformat_tested_function_set = set()
        for dir_path in dir_list:
            py_path_list = [path for path in dir_path.glob('*.py') if path.name != "__init__.py"]
            if py_path_list:
                for py_path in py_path_list:
                    relative_path_to_py_path_from_root_path = get_relative_path(path=py_path, origin_path=self.geoformat_root_path)
                    # if path is not in module_without_unit_test
                    if relative_path_to_py_path_from_root_path not in module_without_unit_test:

                        relative_py_path = Path(relative_path_to_py_path_from_root_path)

                        # get geoformat functions
                        relative_geoformat_lib_py_path_str_for_import = '.'.join(relative_py_path.parts).replace('.py', '')
                        geoformat_function_to_test_set.update(
                            self.get_objects_in_geoformat_module(
                                geoformat_module_import_path=relative_geoformat_lib_py_path_str_for_import,
                                from_list=['object'],
                                object_type=types.FunctionType
                            )
                        )

                        # get tests functions
                        relative_tests_py_path = Path('tests').joinpath(relative_py_path)
                        relative_tests_py_path_parts = list(relative_tests_py_path.parts)
                        relative_tests_py_path_parts[-1] = 'test_' + relative_tests_py_path_parts[-1]
                        relative_tests_py_path_str_for_import = '.'.join(relative_tests_py_path_parts).replace('.py', '')
                        geoformat_tested_function_set.update(
                            self.get_objects_in_geoformat_module(
                                geoformat_module_import_path=relative_tests_py_path_str_for_import,
                                object_type=dict
                            )
                        )

        # filter and only get variable with suffix
        suffix = '_parameters'
        # recreate original name with test function name
        geoformat_tested_function_set = {suffix.join(value.split(suffix)[:-1]) for value in geoformat_tested_function_set if value.endswith(suffix)}
        # make difference to get only not tested function
        not_tested_function_set = geoformat_function_to_test_set - geoformat_tested_function_set
        if not_tested_function_set:
            for not_tested_function_name in not_tested_function_set:
                if not_tested_function_name not in deprecated_function:
                    print('function :', not_tested_function_name, 'must be tested')


if __name__ == '__main__':
    inspector = GeoformatInspector()

    # check tests
    inspector.check_tests_structure()
    inspector.check_tests_function()

    # TODO check if function are in geoformat.__init__
    # TODO check if tested function are in test_all



from geoformat.explore_data.print_data import print_features_data_table, print_metadata_field_table
import time


def measure_time(func):
    def time_it(*args, **kwargs):
        time_started = time.time()
        func(*args, **kwargs)
        time_elapsed = time.time()
        print("{execute} running time is {sec} seconds".format(
            execute=func.__name__,
            sec=round(time_elapsed - time_started, 4)
            )
        )

    return time_it


def compare_geolayer(geolayer_a, geolayer_b):

    # compare this two version
    geolayer_a_metadata = print_metadata_field_table(geolayer_a)
    geolayer_b_metadata = print_metadata_field_table(geolayer_b)

    print('FIELDS METADATA')

    metadata_list = [geolayer_a_metadata, geolayer_b_metadata]
    lines = [metadata_list[i].splitlines() for i in range(len(metadata_list))]
    for l in zip(*lines):
        print(*l, sep='\t')



    print('ATTRIBUTES')
    for i_feat in geolayer_a['features']:
        geolayer_a_feature = geolayer_a['features'][i_feat]
        geolayer_b_feature = geolayer_b['features'][i_feat]
        if geolayer_a_feature != geolayer_b_feature:
            if geolayer_a_feature.get('attributes') != geolayer_b_feature.get('attributes'):
                print(i_feat, 'attributes_diff')
                print('\t', geolayer_a_feature.get('attributes'))
                print('\t', geolayer_b_feature.get('attributes'))
            if geolayer_a_feature.get('geometry') != geolayer_b_feature.get('geometry'):
                print(i_feat, 'geometry_diff')
                print('\t', geolayer_a_feature.get('geometry'))
                print('\t', geolayer_b_feature.get('geometry'))